@@ -379,6 +379,15 @@ class TestNoIPChange(BaseTestCase):
         mock_loads.return_value = mock_config
         undercloud._validate_no_ip_change()
 
+    @mock.patch('instack_undercloud.undercloud.open')
+    @mock.patch('os.path.isfile', return_value=True)
+    def test_update_empty(self, mock_isfile, mock_open):
+        # This would be a way to disable os-net-config from running
+        mock_open.side_effect = [
+            mock.mock_open(read_data='').return_value,
+        ]
+        undercloud._validate_no_ip_change()
+
     @mock.patch('instack_undercloud.undercloud.open')
     @mock.patch('json.loads')
     @mock.patch('os.path.isfile', return_value=True)
@@ -497,9 +506,6 @@ class TestGenerateEnvironment(BaseTestCase):
         self.assertEqual('192.168.24.1', env['LOCAL_IP'])
         # The list is generated from a set, so we can't rely on ordering.
         # Instead make sure that it looks like a valid list by parsing it.
-        drivers = json.loads(env['ENABLED_DRIVERS'])
-        self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
-                                           'pxe_ipmitool'])
         hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
         self.assertEqual(sorted(hw_types), ['idrac', 'ilo', 'ipmi', 'redfish'])
         self.assertEqual(
@@ -551,29 +557,13 @@ class TestGenerateEnvironment(BaseTestCase):
             ['idrac', 'ipmitool', 'no-vendor'])
 
     def test_enabled_discovery(self):
-        self.conf.config(enable_node_discovery=True,
-                         discovery_default_driver='pxe_foobar')
-        env = undercloud._generate_environment('.')
-        # The list is generated from a set, so we can't rely on ordering.
-        # Instead make sure that it looks like a valid list by parsing it.
-        drivers = json.loads(env['ENABLED_DRIVERS'])
-        # Discovery requires enabling the default driver. The pxe_ prefix
-        # designates a classic driver.
-        self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_foobar', 'pxe_ilo',
-                                           'pxe_ipmitool'])
-        self.assertEqual(env['INSPECTION_NODE_NOT_FOUND_HOOK'], 'enroll')
-
-    def test_enabled_hardware_types(self):
         self.conf.config(enable_node_discovery=True,
                          discovery_default_driver='foobar',
                          enabled_hardware_types=['ipmi', 'something'])
         env = undercloud._generate_environment('.')
         # The list is generated from a set, so we can't rely on ordering.
         # Instead make sure that it looks like a valid list by parsing it.
-        drivers = json.loads(env['ENABLED_DRIVERS'])
         hw_types = json.loads(env['ENABLED_HARDWARE_TYPES'])
-        self.assertEqual(sorted(drivers), ['pxe_drac', 'pxe_ilo',
-                                           'pxe_ipmitool'])
         self.assertEqual(sorted(hw_types), ['foobar', 'ipmi', 'something'])
 
     def test_docker_registry_mirror(self):
@@ -583,6 +573,13 @@ class TestGenerateEnvironment(BaseTestCase):
         self.assertEqual('http://foo/bar',
                          env['DOCKER_REGISTRY_MIRROR'])
 
+    def test_docker_insecure_registries(self):
+        self.conf.config(docker_insecure_registries=['http://foo/bar:8787'])
+        env = undercloud._generate_environment('.')
+        insecure_registries = json.loads(env['DOCKER_INSECURE_REGISTRIES'])
+        # Spot check one service
+        self.assertEqual(['http://foo/bar:8787'], insecure_registries)
+
     def test_generate_endpoints(self):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
