@@ -298,7 +298,7 @@ _opts = [
                 default=False,
                 help=('Makes ironic-inspector enroll any unknown node that '
                       'PXE-boots introspection ramdisk in Ironic. By default, '
-                      'the "fake" driver is used for new nodes (it is '
+                      'the "ipmi" driver is used for new nodes (it is '
                       'automatically enabled when this option is set to True).'
                       ' Set discovery_default_driver to override. '
                       'Introspection rules can also be used to specify driver '
@@ -306,10 +306,9 @@ _opts = [
                 ),
     cfg.StrOpt('discovery_default_driver',
                default='ipmi',
-               help=('The default driver or hardware type to use for newly '
-                     'discovered nodes (requires enable_node_discovery set to '
-                     'True). It is automatically added to enabled_drivers '
-                     'or enabled_hardware_types accordingly.')
+               help=('The default hardware type to use for newly discovered '
+                     'nodes (requires enable_node_discovery set to True). '
+                     'It is automatically added to enabled_hardware_types.')
                ),
     cfg.BoolOpt('undercloud_debug',
                 default=True,
@@ -353,6 +352,11 @@ _opts = [
                 help=('Whether to enable docker container images to be build '
                       'on the undercloud.')
                 ),
+    cfg.ListOpt('docker_insecure_registries',
+                default=[],
+                help=('Array of host/port combiniations of docker insecure '
+                      'registries.')
+                ),
     cfg.StrOpt('ipa_otp',
                default='',
                help=('One Time Password to register Undercloud node with '
@@ -375,12 +379,6 @@ _opts = [
                 default=False,
                 help=('Whether to clean overcloud nodes (wipe the hard drive) '
                       'between deployments and after the introspection.')),
-    cfg.ListOpt('enabled_drivers',
-                default=['pxe_ipmitool', 'pxe_drac', 'pxe_ilo'],
-                help=('List of enabled bare metal drivers.'),
-                deprecated_for_removal=True,
-                deprecated_reason=('Please switch to hardware types and '
-                                   'the enabled_hardware_types option.')),
     cfg.ListOpt('enabled_hardware_types',
                 default=['ipmi', 'redfish', 'ilo', 'idrac'],
                 help=('List of enabled bare metal hardware types (next '
@@ -818,16 +816,22 @@ def _validate_no_ip_change():
     need to disallow it early in the install before configurations start to
     be changed.
     """
-    os_net_config_file = '/etc/os-net-config/config.json'
+    if CONF.net_config_override:
+        os_net_config_file = CONF.net_config_override
+    else:
+        os_net_config_file = '/etc/os-net-config/config.json'
     # Nothing to do if we haven't already installed
     if not os.path.isfile(
             os.path.expanduser(os_net_config_file)):
         return
-    with open(os_net_config_file) as f:
-        network_config = json.loads(f.read())
     try:
+        with open(os_net_config_file) as f:
+            network_config = json.loads(f.read())
         ctlplane = [i for i in network_config.get('network_config', [])
                     if i['name'] == 'br-ctlplane'][0]
+    except ValueError:
+        # File was empty
+        return
     except IndexError:
         # Nothing to check if br-ctlplane wasn't configured
         return
@@ -1212,30 +1216,13 @@ def _generate_sysctl_settings():
     return json.dumps(sysctl_settings)
 
 
-def _is_classic_driver(name):
-    """Poor man's way to detect if something is a driver or a hardware type.
-
-    To be removed when we remove support for classic drivers.
-    """
-    return (name == 'fake' or
-            name.startswith('fake_') or
-            name.startswith('pxe_') or
-            name.startswith('agent_') or
-            name.startswith('iscsi_'))
-
-
 def _process_drivers_and_hardware_types(instack_env):
     """Populate the environment with ironic driver information."""
     # Ensure correct rendering of the list and uniqueness of the items
-    enabled_drivers = set(CONF.enabled_drivers)
     enabled_hardware_types = set(CONF.enabled_hardware_types)
     if CONF.enable_node_discovery:
-        if _is_classic_driver(CONF.discovery_default_driver):
-            if CONF.discovery_default_driver not in enabled_drivers:
-                enabled_drivers.add(CONF.discovery_default_driver)
-        else:
-            if CONF.discovery_default_driver not in enabled_hardware_types:
-                enabled_hardware_types.add(CONF.discovery_default_driver)
+        if CONF.discovery_default_driver not in enabled_hardware_types:
+            enabled_hardware_types.add(CONF.discovery_default_driver)
         instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = 'enroll'
     else:
         instack_env['INSPECTION_NODE_NOT_FOUND_HOOK'] = ''
@@ -1268,7 +1255,6 @@ def _process_drivers_and_hardware_types(instack_env):
         if hw_type in enabled_hardware_types:
             vendor_interfaces.add(iface)
 
-    instack_env['ENABLED_DRIVERS'] = _make_list(enabled_drivers)
     instack_env['ENABLED_HARDWARE_TYPES'] = _make_list(enabled_hardware_types)
 
     instack_env['ENABLED_BOOT_INTERFACES'] = _make_list(boot_interfaces)
@@ -1431,12 +1417,21 @@ def _generate_environment(instack_root):
 
     instack_env['SYSCTL_SETTINGS'] = _generate_sysctl_settings()
 
-    if CONF.docker_registry_mirror:
-        instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
-
     instack_env['PUBLIC_INTERFACE_IP'] = instack_env['LOCAL_IP']
     instack_env['LOCAL_IP'] = instack_env['LOCAL_IP'].split('/')[0]
     instack_env['LOCAL_IP_WRAPPED'] = _wrap_ipv6(instack_env['LOCAL_IP'])
+
+    if CONF.docker_registry_mirror:
+        instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
+    if CONF.docker_insecure_registries:
+        instack_env['DOCKER_INSECURE_REGISTRIES'] = json.dumps(
+                CONF.docker_insecure_registries)
+    else:
+        # For backward compatibility with previous defaults
+        instack_env['DOCKER_INSECURE_REGISTRIES'] = json.dumps(
+                [instack_env['LOCAL_IP'] + ':' + '8787',
+                 CONF.undercloud_admin_host + ':' + '8787'])
+
     # We're not in a chroot so this doesn't make sense, and it causes weird
     # errors if it's set.
     if instack_env.get('DIB_YUM_REPO_CONF'):
@@ -1656,7 +1651,7 @@ def _ensure_ssh_selinux_permission():
                         wrong_perm = True
                         break
                 if wrong_perm:
-                    cmd = ['semanage',
+                    cmd = ['sudo', 'semanage',
                            'fcontext', '-a', '-t', 'ssh_home_t',
                            "{}(/.*)?".format(ssh_path)]
                     _run_command(cmd)
@@ -1899,6 +1894,11 @@ def _create_default_plan(mistral, plans, timeout=360):
                                 fail_on_error=True)
 
 
+def _upload_validations_to_swift(mistral):
+    LOG.info('Uploading default validations to Swift')
+    mistral.action_executions.create('tripleo.validations.upload')
+
+
 def _prepare_ssh_environment(mistral):
     mistral.executions.create('tripleo.validations.v1.copy_ssh_key')
 
@@ -1946,6 +1946,7 @@ def _post_config_mistral(instack_env, mistral, swift):
 
     if CONF.enable_validations:
         _prepare_ssh_environment(mistral)
+        _upload_validations_to_swift(mistral)
 
 
 def _migrate_to_convergence(heat):
@@ -2052,12 +2053,10 @@ def _neutron_subnet_create(sdk, network_id, cidr, gateway, host_routes,
     try:
         # DHCP_START contains a ":" then assume a IPv6 subnet
         if ':' in allocation_pool[0]['start']:
-            host_routes = ''
             subnet = sdk.network.create_subnet(
                 name=name,
                 cidr=cidr,
                 gateway_ip=gateway,
-                host_routes=host_routes,
                 enable_dhcp=True,
                 ip_version='6',
                 ipv6_address_mode='dhcpv6-stateless',
@@ -2086,16 +2085,14 @@ def _neutron_subnet_create(sdk, network_id, cidr, gateway, host_routes,
 
 def _neutron_subnet_update(sdk, subnet_id, gateway, host_routes,
                            allocation_pool, name):
+    update_values = {'name': name, 'gateway_ip': gateway,
+                     'host_routes': host_routes,
+                     'allocation_pools': allocation_pool}
     try:
         # DHCP_START contains a ":" then assume a IPv6 subnet
         if ':' in allocation_pool[0]['start']:
-            host_routes = ''
-        subnet = sdk.network.update_subnet(
-                   subnet_id,
-                   name=name,
-                   gateway_ip=gateway,
-                   host_routes=host_routes,
-                   allocation_pools=allocation_pool)
+            del update_values['host_routes']
+        subnet = sdk.network.update_subnet(subnet_id, **update_values)
         LOG.info("Subnet updated %s", subnet)
     except Exception as e:
         LOG.error("Update subnet %s failed: %s", name, e)
@@ -2351,6 +2348,23 @@ def _is_database_upgrade_needed():
 
 def pre_upgrade():
     _configure_logging(DEFAULT_LOG_LEVEL, PATHS.LOG_FILE)
+
+    # Don't upgrade undercloud unless overcloud is in *_COMPLETE.
+    # As we're migrating overcloud stack to convergence in post_config,
+    # which would fail otherwise. It's better to fail fast.
+    user, password, project, auth_url = _get_auth_values()
+    heat = os_client_config.make_client('orchestration',
+                                        auth_url=auth_url,
+                                        username=user,
+                                        password=password,
+                                        project_name=project,
+                                        project_domain_name='Default',
+                                        user_domain_name='Default')
+    for stack in heat.stacks.list():
+        if stack.status != 'COMPLETE':
+            LOG.error('Can not upgrade undercloud with FAILED overcloud')
+            sys.exit(1)
+
     args = ['sudo', 'systemctl', 'stop', 'openstack-*', 'neutron-*',
             'openvswitch', 'httpd']
     LOG.info('Stopping OpenStack and related services')
